/***************************************************************/
//
//
//		DirectX	[ Billboard.cpp ]
//
//											Author	kazuki tanaka
//											Date	2017 2/8
/*---------------------------------------------------------------
Update : 2017/2/8
			Billboard.cpp̍쐬

/*---------------------------------------------------------------
	Include File
---------------------------------------------------------------*/


//#include <Windows.h>
//#include <stdio.h>
//#include <d3dx9.h>

#include "main.h"
#include "Billboard.h"

#include "photo.h"

#include "game_system.h"

#ifdef _DEBUG
#include "d_manager.h"
#endif // _DEBUG

//#pragma comment (lib,"d3d9.lib")
//#pragma comment (lib,"d3dx9.lib")
//#pragma comment (lib,"dxguid.lib")


/*---------------------------------------------------------------
	Macro Difinition
---------------------------------------------------------------*/

#define SAFE_RELEASE(p) { if (p) { (p)->Release(); (p)=NULL; } }

#define FILE_NAME "data/TEXTURE/draw00.png"

/*---------------------------------------------------------------
	Const Number Difinition
---------------------------------------------------------------*/

#define NUM_VERTEX  ( 4 )					
#define NUM_POLYGON ( 2 )					

/*----------------------------------------------------------------
	static member variable Initialize 
----------------------------------------------------------------*/

CBillboard* CBillboard::pBillboard[ BILLBOARD_MAX ] = { };



// CBillboard Constructor
CBillboard::CBillboard( )
{

	// Menber Clear Process
	D3DXMatrixIdentity( &worldMtx );
	//pos = D3DXVECTOR3( 0, 0, 0 );	
	pos = nullptr;
	rot = D3DXVECTOR3( 0, 0, 0 );
	color = D3DXCOLOR( 1, 1, 1, 1 );	
	width = 0;	
	height = 0;	
	u = 0;		
	v = 0;		

	pTexture = nullptr;
	pVtxBuf  = nullptr;

	// Registration Class( Billboard )
	for( int index = 0; index < BILLBOARD_MAX; index++ ){
		if( pBillboard[index] == NULL ){
			pBillboard[index] = this;
			myNumber = index;
			break;
		}
	}


}

// CBillboard Destructor
CBillboard::~CBillboard( )
{

	// Do Nothing!!

}

// CBillboard Create Instance 
CBillboard* CBillboard::Create( const char* BILLBOARD_NAME )
{

	// Create Class Instance
	CBillboard* pObject;
	pObject = new CBillboard;

	// Setting Class Name for Access
	strcpy( pObject->BillboardName, BILLBOARD_NAME );

	return pObject;

}

// CBillboard Access Outside Function  
CBillboard* CBillboard::Access( const char* BILLBOARD_NAME )
{

	// Search CBillboard Class
	for( int index = 0; index < BILLBOARD_MAX; index ++ ){
		if( pBillboard[index] != NULL ){
			if( strcmp( pBillboard[index]->BillboardName, BILLBOARD_NAME ) == 0 ){

				return pBillboard[index];
				break;
			}
		}
	}

	return 0;

}

// CBillboard Initialize
void CBillboard::Init( const LPD3DXVECTOR3 pos, const float width, const float height )
{

	// Initialize Process
	CBillboard::pos    = (pos);
	CBillboard::width  = width;
	CBillboard::height = height;

	// MakeVertex Process
	HRESULT hRes = CBillboard::MakeVertex( );
	if( FAILED( hRes )){
	
		#ifdef _DEBUG
		CDebugManager::DebugMsg( "ErrorCode : 0x%x", hRes );
		#endif // _DEBUG
		return;
	}

	// TextureLoad
	hRes = TextureLoad( );
	if( FAILED( hRes )){
	
		#ifdef _DEBUG
		CDebugManager::DebugMsg( "ErrorCode : 0x%x", hRes );
		#endif // _DEBUG
		return;
	}


}

// CBillboard Uninitialize
void CBillboard::Uninit( void )
{

	// Uninitialize Process
	SAFE_RELEASE( pTexture )
	SAFE_RELEASE( pVtxBuf )


}

// CBillboard Update
void CBillboard::Update( void )
{

	// Update Process
	VERTEX_3D *pVtx;										// zAhX

	// obt@bNzAhX擾
	pVtxBuf -> Lock( 0, 0, (void**)&pVtx, 0 );	


	{ // _obt@[̐ݒ

		// _W̐ݒ
		SetVertexCoord( pVtx );

		// _F̐ݒ
		SetVertexColor( pVtx );

		// _UVl̐ݒ
		SetVertexTexUV( pVtx );

		// @̐ݒ
		SetVertexNormal( pVtx );


	} // ->VTX SETTING END

	// obt@̃AbN
	pVtxBuf->Unlock( );


}

// CBillboard Draw
void CBillboard::Draw( void )
{


	// Draw Process
	LPDIRECT3DDEVICE9 pDevice = GetDevice( );

	// Wϊ[hs
	D3DXMATRIX mtxTranslation;
	D3DXMATRIX mtxRotarion;

	// Cg
	pDevice ->SetRenderState( D3DRS_LIGHTING, false );

	// Xg[
	pDevice->SetStreamSource(0,					
		pVtxBuf,								
		0,													
		sizeof( VERTEX_3D ));					

	// _tH[}bg̐ݒ
	pDevice->SetFVF( FVF_VERTEX_3D );

	D3DXMatrixTranslation( &mtxTranslation, pos->x, pos->y, pos->z );

	worldMtx = CPhoto::GetMtxView( ) * mtxTranslation;

	// foCXɃ[hϊsݒ
	pDevice ->SetTransform( D3DTS_WORLD, &worldMtx );

	// eNX`̐ݒ
	pDevice->SetTexture( 0, pTexture );
		
	// `揈
	pDevice ->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, NUM_POLYGON );
		

	pDevice ->SetRenderState( D3DRS_LIGHTING, true );



}

// CBillboard Load Texture Data
HRESULT CBillboard::TextureLoad( void )
{

	LPDIRECT3DDEVICE9 pDevice = GetDevice( );

	// Texture Load Process
	HRESULT hres = D3DXCreateTextureFromFile( pDevice, FILE_NAME, &pTexture );
	if( FAILED( hres )){
		#ifdef _DEBUG
		MessageBox( NULL, "eNX`t@C̓ǂݍ݂Ɏs܂!" , "TEXTURE LOAD ERROR!!", MB_OK | MB_ICONWARNING );
		#endif	// _DEBUG
		return E_FAIL;
	}

	return S_OK;

}

// CBillboard Release Class All
void CBillboard::ReleaseAll( void )
{

	// Release CBillboard All
	for( int index = 0; index < BILLBOARD_MAX; index ++ ){
		if( pBillboard[ index ] ){

			pBillboard[ index ] ->Uninit( );
			SAFE_DELETE( pBillboard[ index ] )
		}
	}


}

// CBillboard Update Class All
void CBillboard::UpdateAll( void )
{

	// Update CBillboard All
	for( int index = 0; index < BILLBOARD_MAX; index ++ ){
		if( pBillboard[ index ] != NULL ){

			pBillboard[ index ] ->Update( );
		}
	}


}

// CBillboard Draw Class All
void CBillboard::DrawAll( void )
{

	// Draw CBillboard All
	for( int index = 0; index < BILLBOARD_MAX; index ++ ){
		if( pBillboard[ index ] != NULL ){

			pBillboard[ index ] ->Draw( );
		}
	}


}

// CBillboard Make Vertex
HRESULT CBillboard::MakeVertex( void )
{

	// foCX̎󂯎
	LPDIRECT3DDEVICE9 pDevice = GetDevice( );

	VERTEX_3D *pVtx;										// zAhX

	if(FAILED(pDevice->CreateVertexBuffer(
		sizeof(VERTEX_3D) * NUM_VERTEX,						// mۂobt@TCY(_m) P:oCg
			D3DUSAGE_WRITEONLY,								// gp@
			FVF_VERTEX_3D,									// gp钸_tH[}bg(ȏȂQƂȂ̂0ł悢)
			D3DPOOL_MANAGED,								// obt@̊Ǘ@(obt@̒ɏ񂾂̂̊Ǘ@)
			&pVtxBuf,										// i[|C^
			NULL)))
	{
		#ifdef _DEBUG
		MessageBox(NULL,"Make Vertex Process Error !! \n Code : Billboard.cpp line 227","vertex buffer ERROR!!",MB_OK);
		#endif	// _DEBUG
		return E_FAIL;
	}


	// obt@bNzAhX擾
	pVtxBuf -> Lock( 0, 0, (void**)&pVtx, 0 );	


	{ // VTX BUFFER SETTING

		// _W̐ݒ
		SetVertexCoord( pVtx );

		// _F̐ݒ
		SetVertexColor( pVtx );

		// _UVl̐ݒ
		SetVertexTexUV( pVtx );

		// @̐ݒ
		SetVertexNormal( pVtx );


	} // ->VTX SETTING END

	// obt@̃AbN
	pVtxBuf->Unlock( );


	return S_OK;

}

// CBillboard Set Vertex info
void CBillboard::SetVertexCoord( VERTEX_3D* pVtx )
{

	D3DXVECTOR3 radius( width*0.5f, height*0.5f, 0.0f );

	// Set Vertex Coord
	pVtx[ 0 ].pos = D3DXVECTOR3( -radius.x,  radius.y, 0.0f );
	pVtx[ 1 ].pos = D3DXVECTOR3(  radius.x,  radius.y, 0.0f );
	pVtx[ 2 ].pos = D3DXVECTOR3( -radius.x, -radius.y, 0.0f );
	pVtx[ 3 ].pos = D3DXVECTOR3(  radius.x, -radius.y, 0.0f );


}

// CBillboard Set Vertex info
void CBillboard::SetVertexTexUV( VERTEX_3D* pVtx )
{

	// Set Vertex Coord
	pVtx[ 0 ].tex = D3DXVECTOR2( 0.0f, 0.0f );
	pVtx[ 1 ].tex = D3DXVECTOR2( 1.0f, 0.0f );
	pVtx[ 2 ].tex = D3DXVECTOR2( 0.0f, 1.0f );
	pVtx[ 3 ].tex = D3DXVECTOR2( 1.0f, 1.0f );


}

// CBillboard Set Vertex info
void CBillboard::SetVertexColor( VERTEX_3D* pVtx )
{


	// Set Vertex Coord
	pVtx[ 0 ].color = D3DXCOLOR( 1, 1, 1, 1 );
	pVtx[ 1 ].color = D3DXCOLOR( 1, 1, 1, 1 );
	pVtx[ 2 ].color = D3DXCOLOR( 1, 1, 1, 1 );
	pVtx[ 3 ].color = D3DXCOLOR( 1, 1, 1, 1 );


}

// CBillboard Set Vertex info
void CBillboard::SetVertexNormal( VERTEX_3D* pVtx )
{

	// Set Vertex Coord
	pVtx[ 0 ].normal = D3DXVECTOR3( 1, 1, 1 );
	pVtx[ 1 ].normal = D3DXVECTOR3( 1, 1, 1 );
	pVtx[ 2 ].normal = D3DXVECTOR3( 1, 1, 1 );
	pVtx[ 3 ].normal = D3DXVECTOR3( 1, 1, 1 );


}















