/***************************************************************/
//
//
//		DirectX	[ Billboard.h ]
//
//											Author	kazuki tanaka
//											Date	2017 2/8
/*---------------------------------------------------------------
Update : 2017/2/8
			Create Header File -> Billboard.h.h 
 			< Add Class Function >
				CBillboard->Constructor
				CBillboard->Destructor
				CBillboard->Create ( static member )
				CBillboard->Access ( static member )
				CBillboard->Init
				CBillboard->Uninit
				CBillboard->Update
				CBillboard->ReleaseAll ( static member )
				CBillboard->UpdateAll  ( static member )
				CBillboard->DrawAll    ( static member )

Update : 2017/2/8



/*-------------------------------------------------------------*/
#ifndef _BILLBOARD_H_

#define _BILLBOARD_H_


#define BILLBOARD_MAX (50)
#define STRING_MAX    (16)


/*---------------------------------------------------------------
	class
---------------------------------------------------------------*/


// class CBillboard Using Billboard
class CBillboard
{
public:
	
	CBillboard( );								// CBillboard constructor
	~CBillboard( );								// CBillboard destructor

	static CBillboard* Create( 					// CBillboard Create Instance Class
		const char* Billboard_NAME );
	static void ReleaseAll( void );				// CBillboard Class Release All 
	static CBillboard* Access(					// CBillboard Class Access Outside Function 
		const char* Billboard_NAME );

	void Init(									// CBillboard Initialize
		const LPD3DXVECTOR3 pos,
		const float width,
		const float height );
	virtual void Uninit( void );				// CBillboard Uninitialize
	virtual void Update( void );				// CBillboard Update
	virtual void Draw( void );					// CBillboard Draw

	static void UpdateAll( void );				// CBillboard Update Class All
	static void DrawAll( void );				// CBillboard Draw   Class All

	virtual HRESULT TextureLoad( void );		// CBillboard Load Texture Data
 
private:

	virtual void SetVertexCoord( VERTEX_3D* pVtx );	// Set Vertex info
	virtual void SetVertexTexUV( VERTEX_3D* pVtx );	// Set Vertex info
	virtual void SetVertexColor( VERTEX_3D* pVtx );	// Set Vertex info
	virtual void SetVertexNormal( VERTEX_3D* pVtx );// Set Vertex info

	// Billboard Member 
	char BillboardName[ STRING_MAX ];			// 
	int  myNumber;								// 

	// static Member
	static CBillboard* pBillboard[ BILLBOARD_MAX ];

protected:

	HRESULT MakeVertex( void );					// Create Vertex Buffer

	LPDIRECT3DTEXTURE9      pTexture;			// Texture Pointer
	LPDIRECT3DVERTEXBUFFER9 pVtxBuf;			// Vertex Buffer Pointer

	D3DXMATRIX  worldMtx;						// 
	LPD3DXVECTOR3 pos;							// 
	D3DXVECTOR3 rot;							// 
	D3DXCOLOR   color;							// 
	float       width;							// 
	float       height;							// 
	float       u;								// 
	float       v;								// 


};


#endif	//	_BILLBOARD_H_


