/***************************************************************/
//
//
//		DirectX	[ userinterface.cpp ]
//
//											Author	kazuki tanaka
//											Date	2017 2/8
/*---------------------------------------------------------------
Update : 2017/2/8
			userinterface.cpp̍쐬

/*---------------------------------------------------------------
	Include File
---------------------------------------------------------------*/


//#include <Windows.h>
//#include <stdio.h>
//#include <d3dx9.h>

#include "main.h"
#include "userinterface.h"

#include "photo.h"

#include "game_system.h"

#ifdef _DEBUG
#include "d_manager.h"
#include "d_font.h"
#include "d_console.h"
#include "d_log.h"
#endif // _DEBUG

//#pragma comment (lib,"d3d9.lib")
//#pragma comment (lib,"d3dx9.lib")
//#pragma comment (lib,"dxguid.lib")


/*---------------------------------------------------------------
	Macro Difinition
---------------------------------------------------------------*/

#define SAFE_RELEASE(p) { if (p) { (p)->Release(); (p)=NULL; } }

#define FILE_NAME "data/TEXTURE/draw00.png"

/*---------------------------------------------------------------
	Const Number Difinition
---------------------------------------------------------------*/

#define NUM_VERTEX  ( 4 )					
#define NUM_POLYGON ( 2 )

/*----------------------------------------------------------------
	static member variable Initialize 
----------------------------------------------------------------*/

CUserInterface* CUserInterface::pUserInterface[ USERINTERFACE_MAX ] = { };



// CUserInterface Constructor
CUserInterface::CUserInterface( )
{

	// Menber Clear Process
	D3DXMatrixIdentity( &worldMtx );
	pos = D3DXVECTOR3( 0, 0, 0 );	
	rot = D3DXVECTOR3( 0, 0, 0 );
	color = D3DXCOLOR( 1, 1, 1, 1 );	
	width = 1.0f;	
	height = 1.0f;	
	u = 0;		
	v = 0;		
	zfar = 0.0f;

	pTexture = nullptr;
	pVtxBuf  = nullptr;

	// Registration Class( UserInterface )
	for( int index = 0; index < USERINTERFACE_MAX; index++ ){
		if( pUserInterface[index] == NULL ){
			pUserInterface[index] = this;
			myNumber = index;
			break;
		}
	}


}

// CUserInterface Destructor
CUserInterface::~CUserInterface( )
{

	// Do Nothing!!

}

// CUserInterface Create Instance 
CUserInterface* CUserInterface::Create( const char* USERINTERFACE_NAME )
{

	// Create Class Instance
	CUserInterface* pObject;
	pObject = new CUserInterface;

	// Setting Class Name for Access
	strcpy( pObject->userinterfaceName, USERINTERFACE_NAME );


	return pObject;

}

// CUserInterface Access Outside Function  
CUserInterface* CUserInterface::Access( const char* USERINTERFACE_NAME )
{

	// Search CUserInterface Class
	for( int index = 0; index < USERINTERFACE_MAX; index ++ ){
		if( pUserInterface[index] != NULL ){
			if( strcmp( pUserInterface[index]->userinterfaceName, USERINTERFACE_NAME ) == 0 ){

				return pUserInterface[index];
				break;
			}
		}
	}

	return 0;

}

// CUserInterface Initialize
void CUserInterface::Init( const LPD3DXVECTOR3 pos ,const float width, const float height, const char* TEXTURE_NAME )
{

	// Initialize Process
	CUserInterface::pos    = (*pos);
	CUserInterface::width  = width;
	CUserInterface::height = height;

	// MakeVertex Process
	HRESULT hRes = CUserInterface::MakeVertex( );
	if( FAILED( hRes )){
	
		#ifdef _DEBUG
		CDebugManager::DebugMsg( "ErrorCode : 0x%x", hRes );
		#endif // _DEBUG
		return;
	}

	// TextureLoad
	hRes = TextureLoad( TEXTURE_NAME );
	if( FAILED( hRes )){
	
		#ifdef _DEBUG
		CDebugManager::DebugMsg( "ErrorCode : 0x%x", hRes );
		#endif // _DEBUG
		return;
	}


}

// CUserInterface Uninitialize
void CUserInterface::Uninit( void )
{

	// Uninitialize Process
	SAFE_RELEASE( pTexture )
	SAFE_RELEASE( pVtxBuf )


}

// CUserInterface Update
void CUserInterface::Update( void )
{

	// Update Process
	VERTEX_3D *pVtx;										// zAhX

	// obt@bNzAhX擾
	pVtxBuf -> Lock( 0, 0, (void**)&pVtx, 0 );	


	{ // _obt@[̐ݒ

		// _W̐ݒ
		//SetVertexCoord( pVtx );

		// _F̐ݒ
		//SetVertexColor( pVtx );

		// _UVl̐ݒ
		//SetVertexTexUV( pVtx );

		// @̐ݒ
		//SetVertexNormal( pVtx );


	} // ->VTX SETTING END

	// obt@̃AbN
	pVtxBuf->Unlock( );


}

// CUserInterface Draw
void CUserInterface::Draw( void )
{


	// Draw Process
	LPDIRECT3DDEVICE9 pDevice = GetDevice( );

	// Wϊ[hs
	D3DXMATRIX mtxTranslation;
	D3DXMATRIX mtxRotarion;

	// Cg
	pDevice ->SetRenderState( D3DRS_LIGHTING, false );

	// Xg[
	pDevice->SetStreamSource(0,
		pVtxBuf,								
		0,													
		sizeof( VERTEX_3D ));					

	// _tH[}bg̐ݒ
	pDevice->SetFVF( FVF_VERTEX_3D );

	D3DXMatrixTranslation( &mtxTranslation, pos.x*zfar, pos.y*zfar, pos.z );

	worldMtx = CPhoto::GetMtxUserInterface( ) * mtxTranslation;

	// foCXɃ[hϊsݒ
	pDevice ->SetTransform( D3DTS_WORLD, &worldMtx );

	// eNX`̐ݒ
	pDevice->SetTexture( 0, pTexture );
		
	// `揈
	pDevice ->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, NUM_POLYGON );
		

	pDevice ->SetRenderState( D3DRS_LIGHTING, true );



}

// CUserInterface Load Texture Data
HRESULT CUserInterface::TextureLoad( const char* TEXTURE_NAME )
{

	LPDIRECT3DDEVICE9 pDevice = GetDevice( );

	// Texture Load Process
	HRESULT hres = D3DXCreateTextureFromFile( pDevice, TEXTURE_NAME, &pTexture );
	if( FAILED( hres )){
		#ifdef _DEBUG
		MessageBox( NULL, "eNX`t@C̓ǂݍ݂Ɏs܂!" , "TEXTURE LOAD ERROR!!", MB_OK | MB_ICONWARNING );
		#endif	// _DEBUG
		return E_FAIL;
	}

	return S_OK;

}

// CUserInterface Release Class All
void CUserInterface::ReleaseAll( void )
{

	// Release CUserInterface All
	for( int index = 0; index < USERINTERFACE_MAX; index ++ ){
		if( pUserInterface[ index ] ){

			pUserInterface[ index ] ->Uninit( );
			SAFE_DELETE( pUserInterface[ index ] )
		}
	}


}

// CUserInterface Update Class All
void CUserInterface::UpdateAll( void )
{

	// Update CUserInterface All
	for( int index = 0; index < USERINTERFACE_MAX; index ++ ){
		if( pUserInterface[ index ] != NULL ){

			pUserInterface[ index ] ->Update( );
		}
	}


}

// CUserInterface Draw Class All
void CUserInterface::DrawAll( void )
{

	// Draw CUserInterface All
	for( int index = 0; index < USERINTERFACE_MAX; index ++ ){
		if( pUserInterface[ index ] != NULL ){

			pUserInterface[ index ] ->Draw( );
		}
	}


}

// CUserInterface Make Vertex
HRESULT CUserInterface::MakeVertex( void )
{

	// foCX̎󂯎
	LPDIRECT3DDEVICE9 pDevice = GetDevice( );

	VERTEX_3D *pVtx;										// zAhX

	if(FAILED(pDevice->CreateVertexBuffer(
		sizeof(VERTEX_3D) * NUM_VERTEX,						// mۂobt@TCY(_m) P:oCg
			D3DUSAGE_WRITEONLY,								// gp@
			FVF_VERTEX_3D,									// gp钸_tH[}bg(ȏȂQƂȂ̂0ł悢)
			D3DPOOL_MANAGED,								// obt@̊Ǘ@(obt@̒ɏ񂾂̂̊Ǘ@)
			&pVtxBuf,										// i[|C^
			NULL)))
	{
		#ifdef _DEBUG
		MessageBox(NULL,"Make Vertex Process Error !! \n Code : userinterface.cpp line 227","vertex buffer ERROR!!",MB_OK);
		#endif	// _DEBUG
		return E_FAIL;
	}


	// obt@bNzAhX擾
	pVtxBuf -> Lock( 0, 0, (void**)&pVtx, 0 );	


	{ // VTX BUFFER SETTING

		// _W̐ݒ
		SetVertexCoord( pVtx );

		// _F̐ݒ
		SetVertexColor( pVtx );

		// _UVl̐ݒ
		SetVertexTexUV( pVtx );

		// @̐ݒ
		SetVertexNormal( pVtx );


	} // ->VTX SETTING END

	// obt@̃AbN
	pVtxBuf->Unlock( );

	// offset Position, already dont need!
	CUserInterface::pos = D3DXVECTOR3( 0, 0, 0 );


	return S_OK;

}

// CUserInterface Set Vertex info
void CUserInterface::SetVertexCoord( VERTEX_3D* pVtx )
{

	// ZClip Far to View Pos
	zfar = 1.0f + pos.z;

	D3DXVECTOR3 radius( width*0.5f*zfar, height*0.5f*zfar, 0.0f );

	#ifdef _DEBUG
	CConsole::DebugLog( " pos : x : %f, y : %f, z : %f  radius : %f \n", pos.x, pos.y, pos.z, radius );
	#endif // _DEBUG

	// Set Vertex Coord
	pVtx[ 0 ].pos = D3DXVECTOR3( -radius.x + pos.x * zfar,  radius.y + pos.y * zfar, zfar );
	pVtx[ 1 ].pos = D3DXVECTOR3(  radius.x + pos.x * zfar,  radius.y + pos.y * zfar, zfar );
	pVtx[ 2 ].pos = D3DXVECTOR3( -radius.x + pos.x * zfar, -radius.y + pos.y * zfar, zfar );
	pVtx[ 3 ].pos = D3DXVECTOR3(  radius.x + pos.x * zfar, -radius.y + pos.y * zfar, zfar );


}

// CUserInterface Set Vertex info
void CUserInterface::SetVertexTexUV( VERTEX_3D* pVtx )
{

	// Set Vertex Coord
	pVtx[ 0 ].tex = D3DXVECTOR2( 0.0f, 0.0f );
	pVtx[ 1 ].tex = D3DXVECTOR2( 1.0f, 0.0f );
	pVtx[ 2 ].tex = D3DXVECTOR2( 0.0f, 1.0f );
	pVtx[ 3 ].tex = D3DXVECTOR2( 1.0f, 1.0f );


}

// CUserInterface Set Vertex info
void CUserInterface::SetVertexColor( VERTEX_3D* pVtx )
{


	// Set Vertex Coord
	pVtx[ 0 ].color = D3DXCOLOR( 1, 1, 1, 1 );
	pVtx[ 1 ].color = D3DXCOLOR( 1, 1, 1, 1 );
	pVtx[ 2 ].color = D3DXCOLOR( 1, 1, 1, 1 );
	pVtx[ 3 ].color = D3DXCOLOR( 1, 1, 1, 1 );


}

// CUserInterface Set Vertex info
void CUserInterface::SetVertexNormal( VERTEX_3D* pVtx )
{

	// Set Vertex Coord
	pVtx[ 0 ].normal = D3DXVECTOR3( 1, 1, 1 );
	pVtx[ 1 ].normal = D3DXVECTOR3( 1, 1, 1 );
	pVtx[ 2 ].normal = D3DXVECTOR3( 1, 1, 1 );
	pVtx[ 3 ].normal = D3DXVECTOR3( 1, 1, 1 );


}















