/***************************************************************/
//
//
//		DirectX	[ userinterface.h ]
//
//											Author	kazuki tanaka
//											Date	2017 2/8
/*---------------------------------------------------------------
Update : 2017/2/8
			Create Header File -> userinterface.h.h 
 			< Add Class Function >
				CUserInterface->Constructor
				CUserInterface->Destructor
				CUserInterface->Create ( static member )
				CUserInterface->Access ( static member )
				CUserInterface->Init
				CUserInterface->Uninit
				CUserInterface->Update
				CUserInterface->ReleaseAll ( static member )
				CUserInterface->UpdateAll  ( static member )
				CUserInterface->DrawAll    ( static member )

Update : 2017/2/8



/*-------------------------------------------------------------*/
#ifndef _USERINTERFACE_H_

#define _USERINTERFACE_H_


#define USERINTERFACE_MAX (50)
#define STRING_MAX (16)


/*---------------------------------------------------------------
	class
---------------------------------------------------------------*/


// class CUserInterface Using UserInterface
class CUserInterface
{
public:
	CUserInterface( );									// CUserInterface constructor
	~CUserInterface( );									// CUserInterface destructor

	static CUserInterface* Create( 						// CUserInterface Create Instance Class
		const char* USERINTERFACE_NAME );
	static void ReleaseAll( void );						// CUserInterface Class Release All 
	static CUserInterface* Access(						// CUserInterface Class Access Outside Function 
		const char* USERINTERFACE_NAME );

	virtual void Init(									// CUserInterface Initialize
		const LPD3DXVECTOR3 pos,
		const float width,
		const float height,
		const char* TEXTURE_NAME );
	virtual void Uninit( void );						// CUserInterface Uninitialize
	virtual void Update( void );						// CUserInterface Update
	virtual void Draw( void );							// CUserInterface Draw

	static void UpdateAll( void );						// CUserInterface Update Class All
	static void DrawAll( void );						// CUserInterface Draw   Class All

	HRESULT TextureLoad(								// CUserInterface Load Texture Data
		const char* TEXTURE_NAME );

private:

	virtual void SetVertexCoord( VERTEX_3D* pVtx );		// Set Vertex info
	virtual void SetVertexTexUV( VERTEX_3D* pVtx );		// Set Vertex info
	virtual void SetVertexColor( VERTEX_3D* pVtx );		// Set Vertex info
	virtual void SetVertexNormal( VERTEX_3D* pVtx );	// Set Vertex info

	// UserInterface Member 
	char userinterfaceName[ STRING_MAX ];				// 
	int  myNumber;										// 

	// static Member
	static CUserInterface* pUserInterface[ USERINTERFACE_MAX ];

protected:

	HRESULT MakeVertex( void );							// Create Vertex Buffer

	LPDIRECT3DTEXTURE9      pTexture;					// Texture Pointer
	LPDIRECT3DVERTEXBUFFER9 pVtxBuf;					// Vertex Buffer Pointer

	D3DXMATRIX  worldMtx;								// 
	D3DXVECTOR3 pos;									// 
	D3DXVECTOR3 rot;									// 
	D3DXCOLOR   color;									// 
	float       width;									// 
	float       height;									// 
	float       u;										// 
	float       v;										// 
	float       zfar;


};


#endif	//	_USERINTERFACE_H_


